<?php

namespace Freshmail\Core\AdminMenu\Pages;

/**
* Abstract Page class for creating admin page.
* If you are overriding some methods make sure to copy action/filter doc hooks!
*/
abstract class AbstractPage
{
  public function __construct()
  {
  }

  /**
  * Renders this page - just echos the page content.
  *
  * @return void
  * @author MS
  */
  abstract public function renderPage();

  /**
   * Returns page setup data - sections and fields arrays.
   * Can't define as const var because of callbacks.
   *
   * @return array
   * @author MS
   */
  abstract protected function getSetup(): array;
  /**
   * getSetup example implementation
   {
   return [
     'subpages' => [
       [
         // see add_submenu_page() parameters
          'parent_slug' => 'freshmail_settings',
          'page_title' => __('New submenu page', 'freshmail'),
          'menu_title' => __('New submenu page', 'freshmail'),
          'capability' => 'manage_options',
          'menu_slug' => 'freshmail_new_page',
          'callback' => [$this, 'renderPage'], //leave it like that, if doing custom stuff implement it in that method
          'position' => null,
          ]
        ],
        'sections' => [
          [
            // see add_settings_section() parameters
            'id' => 'n_example_section_id',
            'title' => __('Section title', 'freshmail'),
            'callback' => [$this, 'renderSettingsSection'],
            'page' => 'example_page', //see settings_fields() and do_settings_sections()
          ],
        ],
        'fields' => [
          [
            // see add_settings_field() for id, label, callback, page, section and args parameters
            // see register_setting() for page and id parameters
            // see settings_fields() and do_settings_sections() for page parameter
            // see resources/views/admin/settings/field.blade.php for other parameters
            'label' => __('Example field', 'freshmail'),
            'type' => 'text',
            'name' => 'n_example_setting_name',
            'id' => 'n_example_setting_name',
            'css_class' => 'example_field',
            'value' => $this->getValue_n_example_setting_name('default_value'),
            'placeholder' => 'QWERTYUIOP12345',
            'callback' => [$this, 'renderSettingsField_n_example_setting_name'],
            'page' => 'example_page',
            'section' => 'n_example_section_id',
            'args' => [],
          ],
        ],
      ];
    }
    */

  /**
   * Adds (submenu) pages
   *
   * @action freshmail/settings/add_subpage
   * @return void
   * @author MS
   */
  public function addPage()
  {
    $default = [// see add_submenu_page() parameters
      'parent_slug' => 'freshmail_settings',
      'page_title' => __('New submenu page', 'freshmail'),
      'menu_title' => __('New submenu page', 'freshmail'),
      'capability' => 'manage_options',
      'menu_slug' => 'freshmail_new_page',
      'callback' => [$this, 'renderPage'],
      'position' => null,
    ];

    $setup = $this->getSetup()['subpages'];
    if (! empty($setup)) {
      foreach ($setup as $subpage) {
        $subpage = wp_parse_args($subpage, $default);
        add_submenu_page(
            $subpage['parent_slug'],
            $subpage['page_title'],
            $subpage['menu_title'],
            $subpage['capability'],
            $subpage['menu_slug'],
            $subpage['callback'],
            $subpage['position']
        );
      }
    }
  }

  /**
   * Returns field setup data
   *
   * @param string $field_id - field id/name
   * @return void
   * @author MS
   */
  private function getFieldSetup(string $field_id = ''): array
  {
    if (! empty($field_id)) {
      $all_fields = $this->getSetup()['fields'];
      $field_ids = array_column($all_fields, 'id');
      $field_index = array_search($field_id, $field_ids);
      if (false !== $field_index) {
        return $all_fields[$field_index];
      }
    }
    return [];
  }

  /**
   * Renders settings field - echos field content
   *
   * @param string $field_id - field id/name
   * @return void
   * @author MS
   */
  private function renderSettingsField(string $field_id = '')
  {
    if (! empty($field_id)) {
      $field = $this->getFieldSetup($field_id);
      include FRERSHMAIL_RESOURCES_PATH . 'views/admin/settings/field.php';
    }
  }

  /**
   * Renders settings section - echos section content BUT NOT fields themselves.
   *
   * @return void
   * @author MS
   */
  private function renderSettingsSection()
  {
    // $data = $this->getFields();
    // echo template('admin.settings.field', $data);
  }

  /**
   * Adds and register settings fields and sections
   *
   * @action freshmail/settings/add_settings_fields
   * @return void
   * @author MS
   */
  public function addSettings()
  {
    $setup = $this->getSetup();
    if (! empty($setup['sections'])) {
      foreach ($setup['sections'] as $section) {
        $default = [
          'id' => 'n_example_section_id',
          'title' => __('Section title', 'freshmail'),
          'callback' => [$this, 'renderSettingsSection'],
          'page' => 'example_page',
        ];
        $section = wp_parse_args($section, $default);
        add_settings_section(
            $section['id'],
            $section['title'],
            $section['callback'],
            $section['page']
        );
      }
    }

    if (! empty($setup['fields'])) {
      foreach ($setup['fields'] as $field) {
        $default = [
          'label' => __('Example field', 'freshmail'),
          'type' => 'text',
          'name' => 'n_example_setting_name',
          'id' => 'n_example_setting_name',
          'css_class' => 'example_field',
          'value' => '',
          'placeholder' => 'QWERTYUIOP12345',
          'callback' => [$this, 'renderSettingsField_n_example_setting_name'],
          'page' => 'example_page',
          'section' => 'n_example_section_id',
          'args' => ['class' => 'settings-field-row'],
        ];
        $field = wp_parse_args($field, $default);
        add_settings_field(
            $field['id'],
            $field['label'],
            $field['callback'],
            $field['page'],
            $field['section'],
            $field['args']
        );
        register_setting($field['page'], $field['id']);
      }
    }
  }

  /**
   * Returns value of given field
   *
   * @param string $field_id field id/name
   * @param mixed $default
   * @return mixed
   * @author MS
   */
  private function getValue(string $field_id = '', $default = false)
  {
    if (! empty($field_id)) {
      return get_option($field_id, $default);
    }
    return null;
  }

  /**
   * Catches calls and runs other methods if name was matched.
   *
   * @param string $name - called method name
   * @param array $arguments - argumets
   * @return mixed
   */
  public function __call($name, $arguments)
  {
    if (false !== strpos($name, 'renderSettingsField_')) {
      $field_id = str_replace('renderSettingsField_', '', $name);
      $this->renderSettingsField($field_id);
    } elseif (false !== strpos($name, 'getValue_')) {
      $field_id = str_replace('getValue_', '', $name);
      if (array_key_exists(0, $arguments)) {
        return $this->getValue($field_id, $arguments[0]);
      } else {
        return $this->getValue($field_id);
      }
    }
  }
}
