<?php
/**
 * Endpoints: Endpoint
 * Generic endpoint class
 *
 * @author MS
 */
namespace Freshmail\Core\Api\Endpoints;

use Freshmail\Core\Api\Models\ApiConfig;

class AbstractEndpoint
{
  protected ApiConfig $api_config;

  public function __construct(ApiConfig $api_config)
  {
    $this->api_config = $api_config;
  }

  /**
   * calls API endpoint
   *
   * @param string $endpoint - endpoint to call, eg. "/2.0/contacts/"
   * @param string $json_payload - json payload
   * @param string $method - HTTP method to use in request
   * @param string $headers - (optional) additional headers
   * @return array - json response with additional http code and message
   * @throws Exception
   * @author MS
   */
  protected function call(string $endpoint, string $json_payload = '', string $method = 'GET', array $headers = []): array
  {
    if (!in_array(strtoupper($method), ['GET', 'POST', 'PUT', 'DELETE']) || empty($endpoint)) {
      return [];
    }

    if (empty($this->api_config->getToken())) {
      throw new \Exception('Freshmail: API token empty. ' . $endpoint, 401);
    }

    //normalize endpoint path
    if ('/' !== substr($endpoint, 0, 1)) {
      $endpoint = '/' . $endpoint;
    }

    $default_headers = [
      'Authorization' => 'Bearer ' . $this->api_config->getToken(),
      'Content-Type' => 'application/json',
      'Accept' => 'application/json',
      'Cache-Control' => 'no-cache',
    ];
    $headers = wp_parse_args($headers, $default_headers);

    $args = [
      'method' => strtoupper($method),
      'headers' => $headers,
      'body' => $json_payload,
    ];
    $response = wp_remote_request(untrailingslashit($this->api_config->getBaseUrl()) . $endpoint, $args);
    if (is_wp_error($response)) {
      throw new \Exception('FreshMail: There was some connection error (is_wp_error) while calling ' . $endpoint, 400);
    }
    if (401 === $response['response']['code']) {
      throw new \Exception('FreshMail: Unauthorized request - check api key', $response['response']['code']);
    }
    if (404 === $response['response']['code']) {
      throw new \Exception('FreshMail: Endpoint not found', $response['response']['code']);
    }

    //catch all other client and server errors
    if (400 <= $response['response']['code'] && 599 >= $response['response']['code'] && 422 !== $response['response']['code']) {
      throw new \Exception('FreshMail: Error - ' . $response['response']['message'] . ' while calling ' . $endpoint, $response['response']['code']);
    }

    $response_data = json_decode($response['body'], true);
    $response_data['http_code'] = $response['response']['code'];
    $response_data['http_message'] = $response['response']['message'];
    return $response_data;
  }
}
