<?php

namespace Freshmail\Core\Api\Models;

use Freshmail\Core\Api\Submodules\EndpointsManager;

class Subscriber
{
  private ?EndpointsManager $endpoints = null;

  private array $custom_fields = [];
  private string $email = '';

  /**
   * @var string list on which subscriber is supposed to be on
   */
  private string $list = '';

  public function __construct(EndpointsManager $endpoints)
  {
    $this->endpoints = $endpoints;
    $this->setList($this->endpoints->getApiConfig()->getCurrentList());
  }

  public function setCustomFields(array $fields): void
  {
    $this->custom_fields = wp_parse_args($fields, $this->custom_fields);
  }

  public function unsetCustomField(string $field_name): void
  {
    if (isset($this->custom_fields[$field_name])) {
      unset($this->custom_fields[$field_name]);
    }
  }

  public function setList(string $list): void
  {
    $this->list = $list;
  }

  public function setEmail(string $email): void
  {
    $this->email = $email;
  }

  /**
   * @param string $list subscriber list to use, if empty list set in settings will be used
   */
  public function update(string $list = '', array $additional_args = [], bool $add_to_list_if_not_there_yet = true): bool
  {
    if (empty($list)) {
      $list = $this->list;
    }

    if (! filter_var($this->email, FILTER_VALIDATE_EMAIL)) {
      error_log('FRESHMAIL ERROR: Could not update subscriber ' . substr_replace($this->email, '(...)', 3, strrpos($this->email, '.') - 5) . ' at list ' . $list . ' - invalid email.');
      return false;
    }

    try {
      do_action('fm/subscriber/before_update_at_list', $list, $this->email, $this->custom_fields, $additional_args);
      $result = $this->endpoints->subscriber->edit($this->email, $list, $this->custom_fields);
      if (! empty($result['errors'])) {
        if ($add_to_list_if_not_there_yet) {
          foreach ($result['errors'] as $error) {
            if (! empty($error['code']) && 1331 === $error['code']) {
              // subscriber not yet on list
              return $this->addToList($list, $additional_args);
            }
          }
        } else {
          return false;
        }
      }
      do_action('fm/subscriber/after_update_at_list', $list, $this->email, $this->custom_fields, $additional_args);
    } catch (\Throwable $th) {
      error_log(
          'FRESHMAIL ERROR: Could not update subscriber ' .
          substr_replace($this->email, '(...)', 3, strrpos($this->email, '.') - 5) .
          ' at list ' .
          $list .
          '. Message: [' .
          $th->getMessage() .
          '] Code: ' .
          $th->getCode() .
          '. In: ' .
          $th->getFile() .
          ':' .
          $th->getLine()
      );
      return false;
    }

    return true;
  }

  /**
   * @param string $list list to which subscrbe, if empty list set in settings will be used
   */
  public function addToList(string $list = '', array $additional_args = []): bool
  {
    if (empty($list)) {
      $list = $this->list;
    }

    if (! filter_var($this->email, FILTER_VALIDATE_EMAIL)) {
      error_log('FRESHMAIL ERROR: Could not add subscriber ' . substr_replace($this->email, '(...)', 3, strrpos($this->email, '.') - 5) . ' to list ' . $list . ' - invalid email.');
      return false;
    }

    try {
      do_action('fm/subscriber/before_add_to_list', $list, $this->email, $this->custom_fields, $additional_args);
      $result = $this->endpoints->subscriber->add($this->email, $list, $this->custom_fields);
      do_action('fm/subscriber/after_add_to_list', $list, $this->email, $this->custom_fields, $additional_args);
      if (! empty($result['errors'])) {
        if (
            1 !== sizeof($result['errors']) ||
            (
              1 === sizeof($result['errors']) &&
              1304 !== intval($result['errors'][0]['code']) //subscriber alreaddy added
            )
        ) {
          return false;
        }
      }
    } catch (\Throwable $th) {
      error_log('FRESHMAIL ERROR: Could not add subscriber ' . substr_replace($this->email, '(...)', 3, strrpos($this->email, '.') - 5) . ' to list ' . $list . '. Message: [' . $th->getMessage() . '] Code: ' . $th->getCode());
      return false;
    }

    return true;
  }

  /**
   * @param string $list list from which to unsubscrbe, if empty list set in settings will be used
   */
  public function unsubscribe(string $list = '', array $additional_args = []): bool
  {
    if (empty($list)) {
      $list = $this->list;
    }

    if (! filter_var($this->email, FILTER_VALIDATE_EMAIL)) {
      error_log('FRESHMAIL ERROR: Could not remove subscriber ' . substr_replace($this->email, '(...)', 3, strrpos($this->email, '.') - 5) . ' from list ' . $list . ' - invalid email.');
      return false;
    }

    try {
      do_action('fm/subscriber/before_unsubscribe_from_list', $list, $this->email, $additional_args);
      $result = $this->endpoints->subscriber->delete($this->email, $list);
      do_action('fm/subscriber/after_unsubscribe_from_list', $list, $this->email, $additional_args);
      if (! empty($result['errors'])) {
        return false;
      }
    } catch (\Throwable $th) {
      error_log('FRESHMAIL ERROR: Could not remove subscriber ' . substr_replace($this->email, '(...)', 3, strrpos($this->email, '.') - 5) . ' to list ' . $list . '. Message: [' . $th->getMessage() . '] Code: ' . $th->getCode());
      return false;
    }

    return true;
  }

  /**
   * @param string $list list which to check, if empty list set in settings will be used
   */
  public function isEnrolled(string $list = '', array $additional_args = []): bool
  {
    if (empty($list)) {
      $list = $this->list;
    }

    if (! filter_var($this->email, FILTER_VALIDATE_EMAIL)) {
      error_log('FRESHMAIL ERROR: Could not get subscriber ' . substr_replace($this->email, '(...)', 3, strrpos($this->email, '.') - 5) . ' from list ' . $list . ' - invalid email.');
      return false;
    }

    try {
      $data = $this->endpoints->subscriber->get($this->email, $list);
      if (! empty($data['errors'])) {
        return false;
      }
    } catch (\Throwable $th) {
      error_log('FRESHMAIL ERROR: Could not get subscriber ' . substr_replace($this->email, '(...)', 3, strrpos($this->email, '.') - 5) . ' to list ' . $list . '. Message: [' . $th->getMessage() . '] Code: ' . $th->getCode());
      return false;
    }

    return true;
  }
}
