<?php
/**
 * Integrations: SMTP
 * Main SMTP class
 *
 * @author MS
 */
namespace Freshmail\Core\Integrations\Smtp;

use Freshmail\Core\Integrations\Integration;
use Freshmail\Core\Integrations\Smtp\Mailer\Mailer;

class Smtp extends Integration
{
  /**
   * @var Mailer|null
   */
  private $mailer = null;

  public function __construct()
  {
    if ($this->isEnabled()) {
      $this->mailer = createClass('Freshmail\Core\Integrations\Smtp\Mailer\Mailer', $this->getMailerSetup());
    }

    add_filter('pre_update_option_' . $this->getOptionName('enabled'), [$this, 'watchEnableChanges'], 10, 3);
  }

  /**
   * Init
   *
   * @action init
   *
   * @author author
   * @return void
   */
  public function init()
  {
    $this->name = _x('Email settings', 'integration name', 'freshmail');
  }

  /**
   * Returns integration sections for settings screen.
   * See AdminMenu and IntegrationsPage core module for more info
   *
   * @return array
   */
  public function getSettingsSections(): array
  {
    return [
      [
        'id' => $this::OPTION_PREFIX . 'smtp_main_settings_section',
        'title' => $this->name, //__('SMTP integration', 'freshmail'),
        'page' => 'freshmail-integrations',
      ],
    ];
  }

  /**
   * Returns integration fields for settings screen.
   * See AdminMenu and IntegrationsPage core module for more info
   *
   * @return array
   */
  public function getSettingsFields(): array
  {
    $return = [
      [
        'label' => __('Server', 'freshmail'),
        'type' => 'hidden',
        'default_value' => 'smtp.freshmail.com',
        'name' => $this->getOptionName('smtp_server'),
        'id' => $this->getOptionName('smtp_server'),
        'css_class' => 'smtp__server-setting-field',
        'placeholder' => '',
        'page' => 'freshmail-integrations',
        'section' => $this::OPTION_PREFIX . 'smtp_main_settings_section',
        'args' => ['class' => 'settings-field-row settings-field-row--smtp_server hide-if-smtp-disabled hide-if-section-disabled settings-field-row--hidden'],
        'readonly' => 'readonly',
      ],
      [
        'label' => __('Port', 'freshmail'),
        'type' => 'hidden',
        'min_value' => 1,
        'max_value' => 65535,
        'default_value' => 465,
        'name' => $this->getOptionName('smtp_port'),
        'id' => $this->getOptionName('smtp_port'),
        'css_class' => 'smtp__port-setting-field',
        'placeholder' => '',
        'page' => 'freshmail-integrations',
        'section' => $this::OPTION_PREFIX . 'smtp_main_settings_section',
        'args' => ['class' => 'settings-field-row settings-field-row--smtp_port hide-if-smtp-disabled hide-if-section-disabled settings-field-row--hidden'],
        'readonly' => 'readonly',
      ],
      [
        'label' => __('Login', 'freshmail'),
        'type' => 'hidden',
        'default_value' => 'smtp@freshmail.com',
        'name' => $this->getOptionName('smtp_login'),
        'id' => $this->getOptionName('smtp_login'),
        'css_class' => 'smtp__login-setting-field',
        'page' => 'freshmail-integrations',
        'section' => $this::OPTION_PREFIX . 'smtp_main_settings_section',
        'args' => ['class' => 'settings-field-row settings-field-row--hidden hide-if-smtp-disabled hide-if-section-disabled'],
        'readonly' => 'readonly',
      ],
      [
        'label' => __('From name', 'freshmail'),
        'type' => 'text',
        'default_value' => 'smtp.freshmail.com',
        'name' => $this->getOptionName('from_name'),
        'id' => $this->getOptionName('from_name'),
        'css_class' => 'smtp__server-setting-field',
        'placeholder' => '',
        'page' => 'freshmail-integrations',
        'section' => $this::OPTION_PREFIX . 'smtp_main_settings_section',
        'args' => ['class' => 'settings-field-row settings-field-row__smtp_from_name hide-if-smtp-disabled hide-if-section-disabled'],
        'default_value' => get_option('blogname'),
      ],
      [
        'label' => __('From mail', 'freshmail'),
        'type' => 'text',
        'default_value' => 'smtp.freshmail.com',
        'name' => $this->getOptionName('from_mail'),
        'id' => $this->getOptionName('from_mail'),
        'css_class' => 'smtp__server-setting-field',
        'placeholder' => '',
        'page' => 'freshmail-integrations',
        'section' => $this::OPTION_PREFIX . 'smtp_main_settings_section',
        'args' => ['class' => 'settings-field-row settings-field-row__smtp_from_mail hide-if-smtp-disabled hide-if-section-disabled'],
        'default_value' => get_option('admin_email'),
        'description' => __('Put here only <a href="https://freshmail.com/help-and-knowledge/help/account-settings/verify-email-address/" target="_blank">verified email address.</a>', 'freshmail'),
      ],
      [
        'label' => __('Send emails from FreshMail’s infrastructure', 'freshmail'),
        'type' => 'checkbox',
        'name' => $this->getOptionName('enabled'),
        'id' => $this->getOptionName('enabled'),
        'css_class' => 'smtp__server-setting-field',
        'placeholder' => '',
        'page' => 'freshmail-integrations',
        'section' => $this::OPTION_PREFIX . 'smtp_main_settings_section',
        'args' => ['class' => 'settings-field-row settings-field-row__smtp_enabled'],
      ],
    ];

    return $return;
  }

  public function getOptionName(string $option): string
  {
    return $this::OPTION_PREFIX . 'smtp_' . $option;
  }

  /**
   * Returns array with mailer setup
   *
   * @return array
   * @author MS
   */
  private function getMailerSetup(): array
  {
    return [
      get_option($this->getOptionName('smtp_server'), ''),
      get_option($this->getOptionName('smtp_port'), ''),
      get_option($this->getOptionName('smtp_login'), ''),
      get_option('f_bearer_token', ''),
      get_option($this->getOptionName('from_mail'), ''),
      get_option($this->getOptionName('from_name'), ''),
    ];
  }

  /**
   * Checks if integration can be enabled.
   *
   * @return bool true if integration can be enabled otherwise false
   * @author MS
   */
  public function canBeEnabled(): bool
  {
    //required data
    $options_to_check = [
      'f_bearer_token',
      $this->getOptionName('smtp_server'),
      $this->getOptionName('smtp_port'),
      $this->getOptionName('smtp_login'),
      $this->getOptionName('from_mail'),
      $this->getOptionName('from_name'),
    ];

    //check if all required data is there
    foreach ($options_to_check as $option_name) {
      if (empty(get_option($option_name, null))) {
        return false;
      }
    }

    return true;
  }

  /**
   * Watches changes of enabled setting.
   * If integration is being enabled checks the needed requirements and enables it.
   * If check fails the new value is not saved.
   * Runs on pre_update_option_{enabled} - see constructor
   *
   * @param mixed  $value     The new, unserialized option value.
   * @param mixed  $old_value The old option value.
   * @param string $option    Option name.
   * @return mixed
   * @author MS
   */
  public function watchEnableChanges($value, $old_value, string $option)
  {
    if (empty($value)) {
      //check only when enabling
      return $value;
    }

    if ($this->canBeEnabled()) {
      return $value;
    } else {
      $message = __('Can\'t enable %s - some of reqired data is missing.', 'freshmail');
      $message = sprintf($message, $this->getName());
      freshmail('notifications')->addNotification($message, 'warning', false);
    }
    return $old_value;
  }

  /**
   * Checks if integration can still work after updating some of its options
   *
   * @action updated_option
   * @param mixed  $old_value The old option value.
   * @param mixed  $value     The new option value.
   * @param string $option    Option name.
   * @return void
   * @author MS
   */
  public function checkIfCanStayEnabled($option, $old_value, $value)
  {
    if (0 === strpos($option, $this::OPTION_PREFIX . 'smtp_')) {
      if (!$this->canBeEnabled()) {
        $this->disable();
      }
    }
  }
}
